/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.managers;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.jena.atlas.io.IndentedWriter;
import org.apache.jena.query.Dataset;
import org.apache.jena.query.DatasetFactory;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.vocabulary.RDF;
import org.apache.jena.vocabulary.RDFS;
import org.slf4j.Logger;

import fr.emse.gitlab.saref.SAREF;
import fr.emse.gitlab.saref.SAREFPipeline;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.SAREFRepositoryErrorLogger;
import fr.emse.gitlab.saref.entities.SAREFRepository;
import fr.emse.gitlab.saref.entities.SAREFTerm;
import fr.emse.gitlab.saref.entities.SAREFVersion;
import fr.emse.gitlab.saref.utils.Languages;
import fr.emse.gitlab.saref.vocabs.EX;

public class TermSiteManager extends SAREFRepositoryErrorLogger {
	
	private static final String REPO = "repo";

	private final static Logger getLogger(SAREFPipeline pipeline, SAREFRepository repository) {
		String repositoryName = repository.getName();
		final String loggerName = SAREF.getMessage(REPO, repositoryName);
		return pipeline.getLogger(loggerName);
	}

	private static enum MESSAGE {
		term_rdf, term_html
	}

	private final File siteDir;
	private final File ontoDir;
	
	
	public TermSiteManager(SAREFPipeline pipeline, SAREFRepository repository) {
		super(pipeline, repository, getLogger(pipeline, repository));
		this.siteDir = new File(pipeline.targetDir, SAREF.NAME_SITE);
		this.ontoDir = new File(siteDir, repository.getPath());
	}
	
	public void generateSite() throws SAREFPipelineException {
		for(SAREFTerm term: repository.getTerms().values()) {
			generateModel(term);
			generateRDFFiles(term);
			generateHTMLFile(term);
		}
	}

	private void generateModel(SAREFTerm term) throws SAREFPipelineException {
		Model model = term.getModel();
		model.add(term.getResource(), RDF.type, RDFS.Resource);
		model.add(term.getResource(), EX.localName, term.getLocalName());
		term.getIsDefinedBy().stream().forEach(v->{
			model.add(term.getResource(), RDFS.isDefinedBy, v.getResource());
		});
		term.getIsUsedBy().stream().forEach(v->{
			model.add(term.getResource(), EX.isUsedBy, v.getResource());
		});		
		SAREFVersion lastVersion = getLastVersion(term);
		model.add(QueryExecutionFactory.create(String.format("DESCRIBE <%s>", term.getIRI()), lastVersion.getModel()).execDescribe());
	}

	private SAREFVersion getLastVersion(SAREFTerm term) {
		SAREFVersion lastVersion = null;
		for(SAREFVersion v : term.getIsDefinedBy()) {
			if(lastVersion == null) {
				lastVersion = v;
				continue;
			}
			if(v.getVersionName().compareTo(lastVersion.getVersionName()) > 0) {
				lastVersion = v;
			}
		}
		return lastVersion;
	}

	private void generateRDFFiles(SAREFTerm term) throws SAREFPipelineException {
		Model model = term.getModel();
		for (Languages l : Languages.values()) {
			String termFileName = term.getFileName(l);
			File file = new File(ontoDir, termFileName);
			try (FileOutputStream fos = new FileOutputStream(file)) {
				model.write(fos, l.getLang());
			} catch (IOException ex) {
				String msg = getMessage(MESSAGE.term_rdf, project, term.getLocalName());
				errorLogger.warn(msg, ex);
			}
		}		
	}

	private void generateHTMLFile(SAREFTerm term) throws SAREFPipelineException {
		Dataset dataset = DatasetFactory.create();
		dataset.addNamedModel(term.getIRI(), term.getModel());
		SAREFVersion lastVersion = term.getRepository().getVersions().lastEntry().getValue();
		dataset.addNamedModel(lastVersion.getIRI(), lastVersion.getModel());
		String htmlFileName = String.format("%s.html", term.getLocalName());
		File file = new File(ontoDir, htmlFileName);
		try (IndentedWriter writer = new IndentedWriter(new FileOutputStream(file))) {
			siteManager.generateTermDocumentation(term, writer, dataset);
		} catch (Exception ex) {
			String msg = getMessage(MESSAGE.term_html, project, term.getLocalName());
			errorLogger.warn(msg, ex);
		}		
	}
	
}
